@@ -9,19 +9,10 @@ import time
 
 import org.wayround.utils.threading
 
-# blockables
-CAT_READWRITE_B = ['file', 'pipe',
-                   'socket', 'ssl', 'pyopenssl']
+CAT_READWRITE_TYPES = ['sync', 'async']
 
-# non-blockables
-CAT_READWRITE_NB = ['pipe-nb',
-                    'socket-nb', 'ssl-nb', 'pyopenssl-nb']
-
-
-CAT_READWRITE_TYPES = CAT_READWRITE_NB + CAT_READWRITE_B
-
-
-SELECTABLE = ['socket-nb', 'ssl-nb', 'pyopenssl-nb', 'pipe-nb']
+SELECT_SLEEP = 0.02
+UNSELECTABLE_SLEEP = SELECT_SLEEP
 
 
 class CatTerminationFlagFound(Exception):
@@ -35,7 +26,9 @@ class Streamer:
             stream_object,
             stream_object_read_meth,
             stream_object_write_meth,
-            stream_object_mode='file',
+            stream_object_mode='sync',
+            stream_object_selectable=True,
+            stream_object_unselectable_sleep=0.2,
             bs=2 * 1024 ** 2,
             descriptor_to_wait_for=None,
             flush_after_each_write=False,
@@ -58,6 +51,9 @@ class Streamer:
         if not standard_write_method_result in [None, False, True]:
             raise ValueError("invalid `standard_write_method_result'")
 
+        if not isinstance(stream_object_selectable, bool):
+            raise TypeError("`stream_object_selectable' must be bool")
+
         self._verbose = verbose
         self._debug = debug
 
@@ -65,6 +61,9 @@ class Streamer:
 
         self._stream_object = stream_object
         self._stream_object_mode = stream_object_mode
+        self._stream_object_selectable = stream_object_selectable
+        self._stream_object_unselectable_sleep = \
+            stream_object_unselectable_sleep
 
         self._stream_object_read_meth = stream_object_read_meth
         self._stream_object_write_meth = stream_object_write_meth
@@ -77,7 +76,9 @@ class Streamer:
 
         self._flush_after_each_write = flush_after_each_write
 
+        self._on_exit_callback = on_exit_callback
         self._on_input_read_error = on_input_read_error
+        self._on_output_write_error = on_output_write_error
 
         self._termination_event = termination_event
 
@@ -94,7 +95,12 @@ class Streamer:
                 )
 
         while len(
-                select.select([self._descriptor_to_wait_for], [], [], 0.2)[0]
+                select.select(
+                    [self._descriptor_to_wait_for],
+                    [],
+                    [],
+                    SELECT_SLEEP
+                    )[0]
                 ) == 0:
 
             if (self._termination_event
@@ -108,24 +114,33 @@ class Streamer:
                         self._descriptor_to_wait_for
                         )
                     )
-            if isinstance(self._stream_object, ssl.SSLSocket):
-                if self._debug:
-                    logging.debug(
-                        "{}: {} pending {}".format(
-                            self._thread_name,
-                            self._stream_object,
-                            self._stream_object.pending()
-                            )
-                        )
 
-                if self._stream_object.pending() != 0:
-                    if self._debug:
-                        logging.debug(
-                            "{}: receiving pending data".format(
-                                self._thread_name
-                                )
-                            )
-                    break
+            # the ssl.SSLSocket.pending() method is undocumented.
+            # as so, I can't use it her, as it can be removed from ssl
+            # socket class object at any time.
+            # Instead of it I'll use time.sleep().
+            # say thanks to /* Antoine Pitrou (pitrou) */
+            # http://bugs.python.org/issue21430
+            # .. commenting out
+
+            # if isinstance(self._stream_object, ssl.SSLSocket):
+            #     if self._debug:
+            #         logging.debug(
+            #             "{}: {} pending {}".format(
+            #                 self._thread_name,
+            #                 self._stream_object,
+            #                 self._stream_object.pending()
+            #                 )
+            #             )
+            #
+            #     if self._stream_object.pending() != 0:
+            #         if self._debug:
+            #             logging.debug(
+            #                 "{}: receiving pending data".format(
+            #                     self._thread_name
+            #                     )
+            #                 )
+            #         break
 
         if self._debug:
             logging.debug(
@@ -148,7 +163,12 @@ class Streamer:
                 )
 
         while len(
-                select.select([], [self._descriptor_to_wait_for], [], 0.2)[1]
+                select.select(
+                    [],
+                    [self._descriptor_to_wait_for],
+                    [],
+                    SELECT_SLEEP
+                    )[1]
                 ) == 0:
 
             if (self._termination_event
@@ -173,7 +193,7 @@ class Streamer:
 
         return
 
-    def read(self):
+    def read(self, data_size=None):
         """
         ret[0] == True - stream closed
         """
@@ -183,7 +203,7 @@ class Streamer:
 
         try:
 
-            if self._stream_object_mode in ['file', 'pipe', 'socket', 'ssl']:
+            if self._stream_object_mode == 'sync':
 
                 if (self._termination_event
                         and self._termination_event.is_set()):
@@ -218,12 +238,13 @@ class Streamer:
                     ret_buff = res
                     res = None
 
-            elif (self._stream_object_mode
-                  in ['socket-nb', 'ssl-nb', 'pipe-nb']):
+            elif self._stream_object_mode == 'async':
 
-                if self._stream_object_mode in SELECTABLE:
+                if self._stream_object_selectable:
                     if self._descriptor_to_wait_for is not None:
                         self._wait_input_avail()
+                else:
+                    time.sleep(self._stream_object_unselectable_sleep)
 
                 while True:
 
@@ -232,28 +253,50 @@ class Streamer:
                         raise CatTerminationFlagFound()
 
                     try:
-                        ret_buff = self._stream_object_read_meth(self._bs)
+                        if data_size is None:
+                            ret_buff = self._stream_object_read_meth(self._bs)
+                        else:
+                            if data_size > self._bs:
+                                ret_buff = self._stream_object_read_meth(
+                                    self._bs
+                                    )
+                            else:
+                                ret_buff = self._stream_object_read_meth(
+                                    data_size
+                                    )
 
                     except BlockingIOError:
                         pass
 
                     except ssl.SSLWantReadError:
-                        logging.debug("ssl.SSLWantReadError")
-                        select.select(
-                            [self._descriptor_to_wait_for],
-                            [],
-                            [],
-                            0.2
-                            )
+                        # this is commented out, because withous use of
+                        # .pending() too many garbage debugging messages
+                        # are printed
+                        # logging.debug("ssl.SSLWantReadError")
+                        if self._stream_object_selectable:
+                            select.select(
+                                [self._descriptor_to_wait_for],
+                                [],
+                                [],
+                                SELECT_SLEEP
+                                )
+                        else:
+                            time.sleep(self._stream_object_unselectable_sleep)
 
                     except ssl.SSLWantWriteError:
-                        logging.debug("ssl.SSLWantWriteError")
-                        select.select(
-                            [],
-                            [self._descriptor_to_wait_for],
-                            [],
-                            0.2
-                            )
+                        # this is commented out, because withous use of
+                        # .pending() too many garbage debugging messages
+                        # are printed
+                        # logging.debug("ssl.SSLWantWriteError")
+                        if self._stream_object_selectable:
+                            select.select(
+                                [],
+                                [self._descriptor_to_wait_for],
+                                [],
+                                SELECT_SLEEP
+                                )
+                        else:
+                            time.sleep(self._stream_object_unselectable_sleep)
                     else:
                         break
 
@@ -320,9 +363,11 @@ class Streamer:
 
                 while len(buff) != 0:
 
-                    if self._stream_object_mode in SELECTABLE:
+                    if self._stream_object_selectable:
                         if self._descriptor_to_wait_for is not None:
                             self._wait_output_avail()
+                    else:
+                        time.sleep(self._stream_object_unselectable_sleep)
 
                     sb = buff[:self._bs]
                     buff = buff[self._bs:]
@@ -375,33 +420,62 @@ class Streamer:
         return
 
 
+def _is_cat_cycles_counter_enabled(verbose, count):
+    return verbose or count is not None
+
+
+def _is_cat_transferred_data_size_counter_enabled(
+        verbose,
+        maximum_total_data_transfer_size
+        ):
+    return verbose or maximum_total_data_transfer_size is not None
+
+
 def cat(
         stdin,
         stdout,
+
         bs=2 * 1024 ** 2,
         count=None,
+        maximum_total_data_transfer_size=None,
+
         threaded=False,
         thread_name='Thread',
-        verbose=False,
+
         convert_to_str=None,
+
         read_method_name='read',
         write_method_name='write',
-        read_type='file',
-        write_type='file',
+
+        read_type='sync',
+        read_selectable=True,
+        read_unselectable_sleep=UNSELECTABLE_SLEEP,
+
+        write_type='sync',
+        write_selectable=False,
+        write_unselectable_sleep=UNSELECTABLE_SLEEP,
+
         exit_on_input_eof=True,
         flush_after_each_write=False,
         flush_on_input_eof=False,
         close_output_on_eof=False,
+
         descriptor_to_wait_for_input=None,
         descriptor_to_wait_for_output=None,
+
         apply_input_seek=True,
         apply_output_seek=True,
+
         standard_write_method_result=True,
+
         termination_event=None,
+
+        verbose=False,
+        debug=False,
+
         on_exit_callback=None,
         on_input_read_error=None,
-        on_output_write_error=None,
-        debug=False
+        on_output_write_error=None
         ):
 
     if not read_method_name.isidentifier():
@@ -411,10 +485,20 @@ def cat(
         raise ValueError("Wrong `write_method_name' parameter")
 
     if not read_type in CAT_READWRITE_TYPES:
-        raise ValueError("`read_type' must be file or socket")
+        raise ValueError(
+            "`read_type' must be in {}, not {}".format(
+                CAT_READWRITE_TYPES,
+                read_type
+                )
+            )
 
     if not write_type in CAT_READWRITE_TYPES:
-        raise ValueError("`write_type' must be file or socket")
+        raise ValueError(
+            "`write_type' must be in {}, not {}".format(
+                CAT_READWRITE_TYPES,
+                write_type
+                )
+            )
 
     if not hasattr(stdin, read_method_name):
         raise ValueError(
@@ -432,8 +516,8 @@ def cat(
         convert_to_str = None
 
     if (convert_to_str is not None
-        and not isinstance(convert_to_str, str)
-        ):
+            and not isinstance(convert_to_str, str)
+            ):
         raise ValueError(
             "convert_to_str can only be str(encoding name), bool or None"
             )
@@ -456,7 +540,11 @@ def cat(
                 read_method_name=read_method_name,
                 write_method_name=write_method_name,
                 read_type=read_type,
+                read_selectable=read_selectable,
+                read_unselectable_sleep=read_unselectable_sleep,
                 write_type=write_type,
+                write_selectable=write_selectable,
+                write_unselectable_sleep=write_unselectable_sleep,
                 exit_on_input_eof=exit_on_input_eof,
                 flush_after_each_write=flush_after_each_write,
                 flush_on_input_eof=flush_on_input_eof,
@@ -483,14 +571,20 @@ def cat(
 
     buff = None
 
-    c = 0
-    bytes_counter = 0
+    if _is_cat_cycles_counter_enabled(verbose, count):
+        c = 0
+
+    if _is_cat_transferred_data_size_counter_enabled(
+            verbose,
+            maximum_total_data_transfer_size
+            ):
+        bytes_counter = 0
 
     if apply_input_seek and hasattr(stdin, 'seek'):
         try:
             stdin.seek(0)
         except:
-            pass
+            logging.exception("Can't seek")
 
     read_method = getattr(stdin, read_method_name)
     write_method = getattr(stdout, write_method_name)
@@ -500,6 +594,8 @@ def cat(
         read_method,
         None,
         stream_object_mode=read_type,
+        stream_object_selectable=read_selectable,
+        stream_object_unselectable_sleep=read_unselectable_sleep,
         bs=bs,
         descriptor_to_wait_for=descriptor_to_wait_for_input,
         thread_name='Input Streamer {}'.format(thread_name),
@@ -513,6 +609,8 @@ def cat(
         None,
         write_method,
         stream_object_mode=write_type,
+        stream_object_selectable=write_selectable,
+        stream_object_unselectable_sleep=write_unselectable_sleep,
         bs=bs,
         descriptor_to_wait_for=descriptor_to_wait_for_output,
         thread_name='Output Streamer {}'.format(thread_name),
@@ -528,7 +626,15 @@ def cat(
             if termination_event and termination_event.is_set():
                 raise CatTerminationFlagFound()
 
-            closed, buff = s1.read()
+            if maximum_total_data_transfer_size is None:
+                closed, buff = s1.read()
+            else:
+                if bytes_counter + bs > maximum_total_data_transfer_size:
+                    closed, buff = s1.read(
+                        maximum_total_data_transfer_size - bytes_counter
+                        )
+                else:
+                    closed, buff = s1.read()
 
             if not closed and buff is not None and len(buff) != 0:
 
@@ -567,13 +673,26 @@ def cat(
                 if exit_on_input_eof:
                     break
 
-            c += 1
+            if _is_cat_cycles_counter_enabled(verbose, count):
+                c += 1
 
             if isinstance(buff, (bytes, str)):
-                if isinstance(buff, bytes):
-                    bytes_counter += len(buff)
-                if isinstance(buff, str):
-                    bytes_counter += len(bytes(buff, 'utf-8'))
+                if _is_cat_transferred_data_size_counter_enabled(
+                        verbose,
+                        maximum_total_data_transfer_size
+                        ):
+
+                    if isinstance(buff, bytes):
+                        bytes_counter += len(buff)
+                    if isinstance(buff, str):
+                        bytes_counter += len(bytes(buff, 'utf-8'))
+
+            if count is not None and count == c:
+                break
+
+            if (maximum_total_data_transfer_size is not None
+                    and bytes_counter >= maximum_total_data_transfer_size):
+                break
 
     except CatTerminationFlagFound:
         if debug:
@@ -599,9 +718,6 @@ def cat(
         if verbose:
             logging.info(" {}: Closing thread stdout".format(thread_name))
 
-#        if write_type in CAT_READWRITE_NB:
-#            stdout.write(b'')
-
         stdout.close()
 
     if verbose:
@@ -612,14 +728,15 @@ Ending `{name}' thread
        {size} bytes ({sizem:4.2f} MiB) transferred,
        with buffer size {bufs} bytes ({bufm:4.2f} MiB)
     }}
-""".format_map({
-            'name': thread_name,
-            'num': c,
-            'size': bytes_counter,
-            'sizem': (float(bytes_counter) / 1024 / 1024),
-            'bufs': bs,
-            'bufm': (float(bs) / 1024 / 1024)
-            }
+""".format_map(
+            {
+                'name': thread_name,
+                'num': c,
+                'size': bytes_counter,
+                'sizem': (float(bytes_counter) / 1024 / 1024),
+                'bufs': bs,
+                'bufm': (float(bs) / 1024 / 1024)
+                }
             )
             )
 
@@ -632,7 +749,12 @@ Ending `{name}' thread
     return
 
 
-def lbl_write(stdin, stdout, threaded=False, typ='info'):
+def lbl_write(
+        stdin,
+        stdout,
+        threaded=False,
+        typ='info'
+        ):
 
     if not typ in ['info', 'error', 'warning']:
         raise ValueError("Wrong `typ' value")
@@ -774,9 +896,9 @@ class SocketStreamer:
 
         if self._stat_threads() == 'stopped':
 
-            sock_type = 'socket-nb'
+            sock_selectable = True
             if self.is_ssl_working():
-                sock_type = 'ssl-nb'
+                sock_selectable = False
 
             self._in_thread = cat(
                 stdin=self._strin,
@@ -788,8 +910,11 @@ class SocketStreamer:
                 bs=self._socket_transfer_size,
                 convert_to_str=None,
                 read_method_name='read',
-                read_type='pipe-nb',
-                write_type=sock_type,
+                read_type='async',
+                read_selectable=True,
+                write_type='async',
+                write_selectable=sock_selectable,
+                write_unselectable_sleep=UNSELECTABLE_SLEEP,
                 exit_on_input_eof=True,
                 descriptor_to_wait_for_input=self._strin,
                 descriptor_to_wait_for_output=self.socket,
@@ -813,8 +938,11 @@ class SocketStreamer:
                 bs=self._socket_transfer_size,
                 convert_to_str=None,
                 read_method_name='recv',
-                read_type=sock_type,
-                write_type='pipe-nb',
+                read_type='async',
+                read_selectable=sock_selectable,
+                read_unselectable_sleep=UNSELECTABLE_SLEEP,
+                write_type='async',
+                write_selectable=True,
                 exit_on_input_eof=True,
                 descriptor_to_wait_for_input=self.socket,
                 descriptor_to_wait_for_output=self._strout,
@@ -983,12 +1111,12 @@ class SocketStreamer:
 
                     except ssl.SSLWantReadError:
                         select.select(
-                            [socket_wrap_result.fileno()], [], [], 0.2
+                            [socket_wrap_result.fileno()], [], [], SELECT_SLEEP
                             )
 
                     except ssl.SSLWantWriteError:
                         select.select(
-                            [], [socket_wrap_result.fileno()], [], 0.2
+                            [], [socket_wrap_result.fileno()], [], SELECT_SLEEP
                             )
 
                     except:
@@ -1120,7 +1248,7 @@ compression:
                 )
             if self.stat() == what:
                 break
-            time.sleep(0.2)
+            time.sleep(SELECT_SLEEP)
 
         return
 
@@ -1140,7 +1268,7 @@ compression:
                 )
             if self._stat_threads() == what:
                 break
-            time.sleep(0.2)
+            time.sleep(SELECT_SLEEP)
 
         return
 
@@ -1184,7 +1312,9 @@ compression:
 
         stopped_by_flag = False
 
-        while len(select.select([], [self.socket.fileno()], [], 0.2)[1]) == 0:
+        while len(
+                select.select([], [self.socket.fileno()], [], SELECT_SLEEP)[1]
+                ) == 0:
 
             if self._stop_flag:
                 stopped_by_flag = True
